"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.until = exports.flatten = exports.complement = exports.intersect = exports.union = exports.unique = exports.toggle = exports.add = exports.append = exports.has = exports.last = exports.first = void 0;
/** Get the first item in the array */
function first(arr) {
    return arr[0];
}
exports.first = first;
/** Get the last item in the array */
function last(arr) {
    return arr[arr.length - 1];
}
exports.last = last;
/** Check if the list includes the item */
function has(list = [], item) {
    // @ts-ignore
    const check = list.has || list.includes;
    return check.call(list, item);
}
exports.has = has;
/** Add the items to the list */
function append(list, items) {
    // @ts-ignore
    const add = list.add || list.push;
    for (const item of items) {
        add.call(list, item);
    }
    return list;
}
exports.append = append;
/** Add the item(s) to the list */
function add(list, ...items) {
    return append(list, items);
}
exports.add = add;
/** Add or remove the item within the set based on whether a flag is truthy */
function toggle(list, value, flag) {
    if (Array.isArray(value)) {
        for (const v of value) {
            toggle(list, v, flag);
        }
        return list;
    }
    if (flag) {
        list.add(value);
    }
    else {
        list.delete(value);
    }
    return list;
}
exports.toggle = toggle;
/** Get a new array with only truthy and non-duplicates returned */
function unique(list) {
    return Array.from(new Set(list.filter((i) => i)).values());
}
exports.unique = unique;
/**
 * Get the distinct elements from both the lists.
 * Given two sets A and B, their union is the set consisting of all objects which are elements of A or of B or of both (see axiom of union). It is denoted by A ∪ B.
 * https://en.wikipedia.org/wiki/Naive_set_theory
 * @param list The left subset.
 * @param include The right subset.
 * @returns the superset/combination of both lists, without redundancy.
 */
function union(list, include) {
    return Array.from(new Set(list.concat(include)).values());
}
exports.union = union;
/**
 * Get the elements from the list that only also exist within the other list.
 * The intersection of A and B is the set of all objects which are both in A and in B. It is denoted by A ∩ B.
 * https://en.wikipedia.org/wiki/Naive_set_theory
 * @param list The left set to intersect.
 * @param allowlist The right set to intersect.
 * @returns the intersection, the values that are within both sets
 */
function intersect(list, allowlist) {
    return list.filter((value) => allowlist.includes(value));
}
exports.intersect = intersect;
/**
 * Get the list without elements from the other list.
 * Finally, the relative complement of B relative to A, also known as the set theoretic difference of A and B, is the set of all objects that belong to A but not to B. It is written as A \ B or A − B.
 * https://en.wikipedia.org/wiki/Naive_set_theory
 * @param list The superset.
 * @param blacklist The excluded subset.
 */
function complement(list, blacklist) {
    return list.filter((value) => blacklist.includes(value) === false);
}
exports.complement = complement;
/** Flatten the array, merging any nested array no matter the depth, into a single array. */
function flatten(list) {
    const result = [];
    for (const item of list) {
        if (Array.isArray(item)) {
            result.push(...flatten(item));
        }
        else {
            result.push(item);
        }
    }
    return result;
}
exports.flatten = flatten;
/** Fetch every item in the list, until a particular one. */
function until(list, until, inclusive = false) {
    const result = [];
    for (const item of list) {
        if (item === until) {
            if (inclusive)
                result.push(item);
            break;
        }
        result.push(item);
    }
    return result;
}
exports.until = until;
