"use strict";
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.until = exports.flatten = exports.complement = exports.intersect = exports.union = exports.unique = exports.toggle = exports.add = exports.append = exports.has = exports.last = exports.first = void 0;
/** Get the first item in the array */
function first(arr) {
    return arr[0];
}
exports.first = first;
/** Get the last item in the array */
function last(arr) {
    return arr[arr.length - 1];
}
exports.last = last;
/** Check if the list includes the item */
function has(list, item) {
    if (list === void 0) { list = []; }
    // @ts-ignore
    var check = list.has || list.includes;
    return check.call(list, item);
}
exports.has = has;
/** Add the items to the list */
function append(list, items) {
    var e_1, _a;
    // @ts-ignore
    var add = list.add || list.push;
    try {
        for (var items_1 = __values(items), items_1_1 = items_1.next(); !items_1_1.done; items_1_1 = items_1.next()) {
            var item = items_1_1.value;
            add.call(list, item);
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (items_1_1 && !items_1_1.done && (_a = items_1.return)) _a.call(items_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return list;
}
exports.append = append;
/** Add the item(s) to the list */
function add(list) {
    var items = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        items[_i - 1] = arguments[_i];
    }
    return append(list, items);
}
exports.add = add;
/** Add or remove the item within the set based on whether a flag is truthy */
function toggle(list, value, flag) {
    var e_2, _a;
    if (Array.isArray(value)) {
        try {
            for (var value_1 = __values(value), value_1_1 = value_1.next(); !value_1_1.done; value_1_1 = value_1.next()) {
                var v = value_1_1.value;
                toggle(list, v, flag);
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (value_1_1 && !value_1_1.done && (_a = value_1.return)) _a.call(value_1);
            }
            finally { if (e_2) throw e_2.error; }
        }
        return list;
    }
    if (flag) {
        list.add(value);
    }
    else {
        list.delete(value);
    }
    return list;
}
exports.toggle = toggle;
/** Get a new array with only truthy and non-duplicates returned */
function unique(list) {
    return Array.from(new Set(list.filter(function (i) { return i; })).values());
}
exports.unique = unique;
/**
 * Get the distinct elements from both the lists.
 * Given two sets A and B, their union is the set consisting of all objects which are elements of A or of B or of both (see axiom of union). It is denoted by A ∪ B.
 * https://en.wikipedia.org/wiki/Naive_set_theory
 * @param list The left subset.
 * @param include The right subset.
 * @returns the superset/combination of both lists, without redundancy.
 */
function union(list, include) {
    return Array.from(new Set(list.concat(include)).values());
}
exports.union = union;
/**
 * Get the elements from the list that only also exist within the other list.
 * The intersection of A and B is the set of all objects which are both in A and in B. It is denoted by A ∩ B.
 * https://en.wikipedia.org/wiki/Naive_set_theory
 * @param list The left set to intersect.
 * @param allowlist The right set to intersect.
 * @returns the intersection, the values that are within both sets
 */
function intersect(list, allowlist) {
    return list.filter(function (value) { return allowlist.includes(value); });
}
exports.intersect = intersect;
/**
 * Get the list without elements from the other list.
 * Finally, the relative complement of B relative to A, also known as the set theoretic difference of A and B, is the set of all objects that belong to A but not to B. It is written as A \ B or A − B.
 * https://en.wikipedia.org/wiki/Naive_set_theory
 * @param list The superset.
 * @param blacklist The excluded subset.
 */
function complement(list, blacklist) {
    return list.filter(function (value) { return blacklist.includes(value) === false; });
}
exports.complement = complement;
/** Flatten the array, merging any nested array no matter the depth, into a single array. */
function flatten(list) {
    var e_3, _a;
    var result = [];
    try {
        for (var list_1 = __values(list), list_1_1 = list_1.next(); !list_1_1.done; list_1_1 = list_1.next()) {
            var item = list_1_1.value;
            if (Array.isArray(item)) {
                result.push.apply(result, __spreadArray([], __read(flatten(item)), false));
            }
            else {
                result.push(item);
            }
        }
    }
    catch (e_3_1) { e_3 = { error: e_3_1 }; }
    finally {
        try {
            if (list_1_1 && !list_1_1.done && (_a = list_1.return)) _a.call(list_1);
        }
        finally { if (e_3) throw e_3.error; }
    }
    return result;
}
exports.flatten = flatten;
/** Fetch every item in the list, until a particular one. */
function until(list, until, inclusive) {
    var e_4, _a;
    if (inclusive === void 0) { inclusive = false; }
    var result = [];
    try {
        for (var list_2 = __values(list), list_2_1 = list_2.next(); !list_2_1.done; list_2_1 = list_2.next()) {
            var item = list_2_1.value;
            if (item === until) {
                if (inclusive)
                    result.push(item);
                break;
            }
            result.push(item);
        }
    }
    catch (e_4_1) { e_4 = { error: e_4_1 }; }
    finally {
        try {
            if (list_2_1 && !list_2_1.done && (_a = list_2.return)) _a.call(list_2);
        }
        finally { if (e_4) throw e_4.error; }
    }
    return result;
}
exports.until = until;
